-- Module to automatically disable records in certain gamemods

local mpr = mpr

-- Reason why was mpr disabled
local reason

local cvars_to_disable = {
    { name = "frontrun_enabled", cv = nil, mapload = true, },
    { name = "elimination", cv = nil, mapload = true, },

--  { name = "hpmod_enabled", cv = nil, }, -- mpr support has been added in hpmod (disabling records when everyone dies) in sunflower's garden

    { name = "fr_enabled", cv = nil, mapload = true, }, -- Friendmod
    
--  { name = "juicebox", cv = nil, }, -- Commented out for tech records
    
    { name = "weathermod", cv = nil, mapload = true, },
    { name = "as_wildtricks", cv = nil, }, -- Acrobasic Wild Tricks
    { name = "as_wildspeed", cv = nil, }, -- Acrobasic Wild Speed
    { name = "paraglider_debug_faytwantedthis", cv = nil, }, -- I don't know but I just added it anyway
    { name = "paraglider_debug_deployanywhere", cv = nil, }, -- Always allow using paragliders
    { name = "paraglider_fullhoryzontalcontrol", cv = nil, }, -- Full control of the paraglider
    { name = "paraglider_fullvertcontrol", cv = nil, }, -- Full control of the paraglider
    { name = "rings", cv = nil, mapload = true, }, -- Rings
}

addHook("ThinkFrame", function()
    if not mpr.isActive() then return end
    
    local should_disable = false
    
    for i = 1, #cvars_to_disable do
        local vardata = cvars_to_disable[i]
    
        local cv = vardata.cv
        
        if (not vardata.mapload or leveltime == 1) and cv and cv.value then
            should_disable = true
            reason = vardata.name
            break
        end
    end
    
    if should_disable then
        mpr.log("Auto-disable mpr because of %s", reason)
        mpr.setActive(false)
    end
end)

local function cvarLookup()
    for _, vardata in ipairs(cvars_to_disable) do
        if not vardata.cv then
            vardata.cv = CV_FindVar(vardata.name)
        end
    end
end

addHook("MapLoad", cvarLookup)
addHook("NetVars", cvarLookup)

addHook("NetVars", function(net)
    reason = net(reason)
end)

addHook("MapLoad", function()
    reason = nil
end)

COM_AddCommand("mpr_status", function(p)
    if mpr.isActive() then
        CONS_Printf(p, "MPR is active")
    elseif reason then
        CONS_Printf(p, "MPR is disabled due to cvar "..reason.." being active")
    else
        CONS_Printf(p, "MPR is disabled due to low player count or because of other mod")
    end
end, COM_LOCAL)