-- Just adds few console commands for managing times. No additional
-- functionality for lua added there

-- Copy-pasting these functions 100th time...

-- Not sure if this thing is portable across platforms (lua reference manual
-- says that char codes aren't. But every encoding have numbers in same order,
-- right?..)
local function isdigit(byte)
    return byte >= string.byte("0") and byte <= string.byte("9")
end

local function getMapId(name)
    local name = name:lower()

    if name:find("map") then
        name = name:sub(4, 5)
    end

    local x = name:byte(1)
    local y = name:byte(2)

    if x == nil or y == nil then return 0 end

    if isdigit(x) then
        if isdigit(y) then
            return (x - string.byte("0")) * 10 + y - string.byte(0)
        end

        return 0
    end

    local q = x - string.byte("a")
    local p

    if isdigit(y) then
        p = y - string.byte("0")
    else
        p = 10 + y - string.byte("a")
    end

    return (36*q + p) + 100
end

COM_AddCommand("mpr_delrecord", function(p, mapname, profile)
    if not mapname then
        CONS_Printf(p, "Usage: mpr_delrecord <map> [profile]")
        return
    end

    local mapid = getMapId(mapname)
    
    if isserver then    
        mpr.setRecord(mapid, nil, profile)

        if mpr.io then mpr.io.saveRecords() end
    end
    
    if gamemap == mapid then
        mpr.sendRecord(mpr.buildEmptyRecord())
    end
    
    CONS_Printf(p, "Record deleted")
end, COM_ADMIN)

COM_AddCommand("mpr_dumprecord", function(p, mapid)
    if p ~= server then return end
    
    if not mapid then mapid = gamemap else mapid = getMapId(mapid) end
    
    if not mapid then
        print("Invalid map id")
        return
    end
    
    print(mpr.dumpRecord(mpr.getRecord(mapid) or mpr.buildEmptyRecord()))
end, COM_LOCAL)

COM_AddCommand("mpr_setprofile", function(p, profile)
    if not profile then
        CONS_Printf(p, "Usage: mpr_setprofile <profile>")
        return
    end

    mpr.setProfile(profile)
    CONS_Printf(p, "Profile '"..profile.."' will be active next race")
end, COM_ADMIN)

COM_AddCommand("mpr_resetprofile", function(p)
    mpr.setProfile(mpr.getDefaultProfile())
    
    CONS_Printf(p, "Default profile will be active next race")
end, COM_ADMIN)

COM_AddCommand("mpr_showprofile", function(p)
    CONS_Printf(p, "Current profile is '"..mpr.getProfile().."'")
end)

COM_AddCommand("mpr_tempdisable", function(p)
    CONS_Printf(p, "Disabled MPR for this race")
    mpr.setActive(false)
end, COM_ADMIN)


COM_AddCommand("mpr_frombackup", function(p, fn, map, profile)
    if not mpr.io then error("mpr.io module not found") end

    if not map then map = "" end

    map = getMapId(map)

    if not (fn ~= nil and map ~= 0 and profile ~= nil) then
        CONS_Printf(p, "Usage: mpr_frombackup <file> <map> <profile>")
        return
    end
    
    mpr.io.loadRecords(fn, map, profile)
    mpr.io.saveRecords()
    
    if gamemap == map then
        mpr.sendRecord(mpr.getRecord(gamemap))
    end
    
    CONS_Printf(p, "Record restored")
end, COM_ADMIN)