-- Module for displaying records

-- O p t i m i z a t i o n
local mpr = mpr

-- Current map record displayed near timer
local display_record

-- New record displayed after race end
local new_record

-- If time for new record is same as for old, display "Tied Record" instead of "New Record"
local new_record_tie = false

local new_record_anim = -50

local cv_record_xoffset = CV_RegisterVar({
    name = "mpr_record_xoffset",
    defaultvalue = 0,
})

local cv_record_yoffset = CV_RegisterVar({
    name = "mpr_record_yoffset",
    defaultvalue = 0,
})

local cv_showrecord = CV_RegisterVar({
    name = "mpr_record_show",
    defaultvalue = "On",
    possiblevalue = CV_OnOff,
})

local cv_shownewrecord = CV_RegisterVar({
    name = "mpr_newrecord_show",
    defaultvalue = "On",
    possiblevalue = CV_OnOff,
})

local cv_forcedisplay = CV_RegisterVar({
    name = "mpr_record_forcedisplay",
    defaultvalue = "Off",
    possiblevalue = CV_OnOff,
})

-- Play gloat sound for every player who got record. With dub support (hehe)
local function playGloat(players)
    if not cv_shownewrecord.value then return end
    
    -- Congrats, you got record while in encore mode!
    if encoremode then
        S_StartSound(nil, sfx_ruby2)
    end

    for _, player in ipairs(players) do
        local dub_sfx = nil
        
        if dub_loaded then dub_sfx = DUB_GetGloatSound(player) end

        if dub_loaded and not player.localskin then
            S_StartSound(nil, dub_sfx)
        else
            local skin = "sonic"

            if player.mo and player.mo.valid then skin = player.localskin or player.mo.skin end

            S_StartSound(nil, skins[skin].soundsid[SKSKPOWR])
        end
    end
end

local function onNewRecord(record, players)
    if not display_record or record.time <= display_record.time then
        new_record = record
        new_record_tie = display_record and (record.time == display_record.time)
        
        for i = 1, #players do
            new_record.players[i].localskin = players[i].localskin
            new_record.players[i].appearance = APPEAR_GetAppearance and APPEAR_GetAppearance(players[i])
        end
        
        new_record_anim = -50
        playGloat(players)
    end
end

mpr.addHook("NewRecord", onNewRecord)

mpr.addHook("RecordLoad", function(record)
    display_record = record
end)

addHook("MapLoad", function() 
    new_record = nil
end)

local cv_colorizedhud, cv_colorizedhudcolor

local function lookupCvars()
    if cv_colorizedhud then return end

    cv_colorizedhud = CV_FindVar("colorizedhud")
    cv_colorizedhudcolor = CV_FindVar("colorizedhudcolor")
end

local function getHudColor()
    if not (cv_colorizedhud and cv_colorizedhud.value) then return end
    
    if cv_colorizedhudcolor.value then return cv_colorizedhudcolor.value end
    
    return (displayplayers[0] or consoleplayer).skincolor
end

addHook("MapLoad", lookupCvars)
addHook("NetVars", lookupCvars)

local cache = {}
local function getPatch(v, name)
    if not cache[name] then cache[name] = v.cachePatch(name) end
    
    return cache[name]
end

local function timeString(time)
    return string.format("%02d", G_TicsToMinutes(time)).."' "..string.format("%02d", G_TicsToSeconds(time)).."\" "..string.format("%02d", G_TicsToCentiseconds(time))
end

local function timeStringTable(time)
    return {
        G_TicsToMinutes(time) / 10,
        G_TicsToMinutes(time) % 10,
        G_TicsToSeconds(time) / 10,
        G_TicsToSeconds(time) % 10,
        G_TicsToCentiseconds(time) / 10,
        G_TicsToCentiseconds(time) % 10,
    }
end

local function scrollName(name)
	local amount = name:len() - 13
	local t = leveltime / (35/min(amount, 17))
	local state = (t / amount) % 4
	
	if state == 0 then
		return name:sub(1, 13)
	elseif state == 1 then
		local advance = t % amount
		return name:sub(1 + advance, 13 + advance)
	elseif state == 2 then
		return name:sub(#name - 12, #name)
	else
		local advance = t % amount
		return name:sub(#name - 12 - advance, #name - advance)
	end
end

-- Stolen from stattracker :p
local function drawCurrentRecord(v)
    --Hide temp stuff
    if not ((mpr.isActive() or cv_forcedisplay.value) and mpr.display.cv_showrecord.value and mpr.isRecordValid(display_record)) then return end
    
    local rgHudOffset = 138
    local screenYSub = 173 - mpr.display.cv_record_yoffset.value
    
    if splitscreen == 0	then
        local scrwidth = v.width()/v.dupx();
        local winheight = v.height()/v.dupy();
        local windiff = ((winheight-200)/2) + (winheight-200) --REEEEEEE DECIMALS ARE THE DEVIL!!!
        local right = ((scrwidth+75)/2) + mpr.display.cv_record_xoffset.value
        local font = "OPPRNK"
        
        local show_holder = display_record.players[1+((leveltime / (TICRATE*5)) % #display_record.players)]
        
        local recordSkin = show_holder.skin
        local recordHolder = show_holder.name
        local recordColor = show_holder.skincolor
		
		-- Fallback if skincolor doesn't exist (record was made on version with extra skin colors)
		if recordColor >= MAXSKINCOLORS then
			recordColor = SKINCOLOR_WHITE
		end
		
        local recordNameLength = string.len(show_holder.name)
        local recordEncore = display_record.encore
        
        local colorchar = ''
        
        if SG_Color2Chat then
            colorchar = SG_Color2Chat[recordColor]
        end
        
        -- TODO - replace this... thing... with some nice "drawTime" function
        local stringTime = timeStringTable(display_record.time)
        
        if leveltime < 138 then
            rgHudOffset = -1 * (138-leveltime)
        else
            rgHudOffset = 0
        end
        
        local patchname = "2STTMBG"
        local colormap = nil
        
        local color = getHudColor()
        
        if color ~= nil then
            patchname = "2SCTMBG"
            colormap = v.getColormap("sonic", color)
        end
        
        v.draw((-19-(rgHudOffset*12))+right, ((winheight-windiff)-screenYSub), getPatch(v, patchname), V_HUDTRANS, colormap)
        if skins[recordSkin] ~= nil then
            v.draw((37-(rgHudOffset*12))+right, ((winheight-windiff)-screenYSub-2), getPatch(v, skins[recordSkin].facemmap), V_HUDTRANS, v.getColormap(recordSkin, recordColor or skins[recordSkin].prefcolor))
        end
        
        if recordEncore then
            v.drawScaled(((37-(rgHudOffset*12))+right+11)<<FRACBITS, (((winheight-windiff)-screenYSub-2))<<FRACBITS+sin(leveltime * ANG10), FRACUNIT/6, getPatch(v, "RUBYICON"))
        end
        
        if show_holder.restat and show_holder.restat.speed ~= -1 then
            v.drawString((37-(rgHudOffset*12))+right, ((winheight-windiff)-screenYSub-2), tostring(show_holder.restat.speed), V_HUDTRANS, "small")
            v.drawString((37-(rgHudOffset*12))+right + 8, ((winheight-windiff)-screenYSub-2) + 8, tostring(show_holder.restat.weight), V_HUDTRANS, "small")
        end
        
        --OPPRNK font only has digits 0-9 so this is printed one at a time
        v.draw((49-(rgHudOffset*12))+right, ((winheight-windiff)-screenYSub), getPatch(v, font.."0"..stringTime[1]), V_HUDTRANS)
        v.draw((55-(rgHudOffset*12))+right, ((winheight-windiff)-screenYSub), getPatch(v, font.."0"..stringTime[2]), V_HUDTRANS)
        v.draw((63-(rgHudOffset*12))+right, ((winheight-windiff)-screenYSub), getPatch(v, "2STMNMK"), V_HUDTRANS)
        v.draw((69-(rgHudOffset*12))+right, ((winheight-windiff)-screenYSub), getPatch(v, font.."0"..stringTime[3]), V_HUDTRANS)
        v.draw((75-(rgHudOffset*12))+right, ((winheight-windiff)-screenYSub), getPatch(v, font.."0"..stringTime[4]), V_HUDTRANS)
        v.draw((83-(rgHudOffset*12))+right, ((winheight-windiff)-screenYSub), getPatch(v, "2STSCMK"), V_HUDTRANS)
        v.draw((89-(rgHudOffset*12))+right, ((winheight-windiff)-screenYSub), getPatch(v, font.."0"..stringTime[5]), V_HUDTRANS)
        v.draw((95-(rgHudOffset*12))+right, ((winheight-windiff)-screenYSub), getPatch(v, font.."0"..stringTime[6]), V_HUDTRANS)
        
        --player name
        if recordNameLength < 7 then
            local nameWidth = v.stringWidth(recordHolder, flags)
            v.drawString((36-(rgHudOffset*12))+(right - nameWidth), ((winheight-windiff)-screenYSub), colorchar..recordHolder, V_HUDTRANS|V_ALLOWLOWERCASE)
        elseif recordNameLength > 13 then
            recordHolder = scrollName(recordHolder)
            local nameWidth = v.stringWidth(recordHolder, flags, "small")
            v.drawString((36-(rgHudOffset*12))+(right - nameWidth), ((winheight-windiff)-screenYSub) + 2, colorchar..recordHolder, V_HUDTRANS|V_ALLOWLOWERCASE, "small")
        else
            local nameWidth = v.stringWidth(recordHolder, flags, "small")
            v.drawString((36-(rgHudOffset*12))+(right - nameWidth), ((winheight-windiff)-screenYSub) + 2, colorchar..recordHolder, V_HUDTRANS|V_ALLOWLOWERCASE, "small")
        end
    end
end

-- Stolen from stattracker too :p
local function drawNewRecord(v)
    if not mpr.cv_maprecords.value or not mpr.display.cv_shownewrecord.value then return end

    if new_record then
        local slideStringTime = timeString(new_record.time)
        
        local show_holder = new_record.players[1+((leveltime / TICRATE) % #new_record.players)]
        
        local slideRecordHolder = show_holder.name
        local slideRecordSkin = show_holder.localskin or show_holder.skin
        local rSkinColorHolder = show_holder.skincolor
        local encore = new_record.encore
        
        if SG_Color2Chat then
            slideRecordHolder = SG_Color2Chat[rSkinColorHolder]..slideRecordHolder
        end
        
        local rgHudOffset = 138
        local screenYSub = 50
        
        local scrwidth = v.width()/v.dupx();
        local winheight = v.height()/v.dupy();
        local windiff = ((winheight-200)/2)
        local right = ((scrwidth+75)/2);
        
        if new_record_anim < 0 then
            new_record_anim = $ + 2
        end
    
        if skins[slideRecordSkin] ~= nil then
			local facewant = skins[slideRecordSkin].facewant
			
			-- Uuuh yea this definitely looks horrible
			if APPEAR_HUD and APPEAR_HUD[show_holder.skin] and APPEAR_HUD[show_holder.skin][show_holder.appearance] then
				facewant = APPEAR_HUD[show_holder.skin][show_holder.appearance].facewant or facewant
			end
			
            v.draw((65-(new_record_anim*5))+right, ((winheight-windiff)-screenYSub), v.cachePatch(facewant), flags, v.getColormap(slideRecordSkin, rSkinColorHolder or skins[slideRecordSkin].prefcolor))
        end
        
        if encore then
            v.drawScaled(((65-(new_record_anim*5))+right + 32)<<FRACBITS,
                         ((winheight-windiff)-screenYSub)<<FRACBITS+4*sin(leveltime * ANG10), FRACUNIT/2, getPatch(v, "RUBYICON"))
        end
        
        if show_holder.restat and show_holder.restat.speed ~= -1 then
            v.drawString((65-(new_record_anim*5))+right+1, ((winheight-windiff)-screenYSub)+1, tostring(show_holder.restat.speed), 0)
            v.drawString((65-(new_record_anim*5))+right+23, ((winheight-windiff)-screenYSub)+23, tostring(show_holder.restat.weight), 0)
        end
        
        local headerWidth = v.stringWidth(new_record_tie and "TIED RECORD" or "NEW RECORD", flags)
        v.drawString((64-(new_record_anim*5))+(right - headerWidth), ((winheight-windiff)-48), new_record_tie and "\x82TIED RECORD" or "\x82NEW RECORD", flags)
        
        local nameWidth = v.stringWidth(slideRecordHolder, flags)
        v.drawString((64-(new_record_anim*5))+(right - nameWidth), ((winheight-windiff)-38), slideRecordHolder, flags)
        
        local timeWidth = v.stringWidth(slideStringTime, flags)
        v.drawString((64-(new_record_anim*5))+(right - timeWidth), ((winheight-windiff)-28), slideStringTime, flags)
    end
end

local function drawHud(v)
    mpr.display.drawCurrentRecord(v)
    mpr.display.drawNewRecord(v)
end

addHook("NetVars", function(net)
    display_record = net(display_record)
    
    if not isserver then
        new_record = nil
    end
end)

mpr.display = {
    -- Doing it like that so draw functions and cvars can be replaced
    cv_record_xoffset = cv_record_xoffset,
    cv_record_yoffset = cv_record_yoffset,
    cv_showrecord = cv_showrecord,
    cv_shownewrecord = cv_shownewrecord,
    drawCurrentRecord = drawCurrentRecord,
    drawNewRecord = drawNewRecord,
    drawHud = drawHud,
}

hud.add(function(...)
    mpr.display.drawHud(...)
end)

--[[
COM_AddCommand("mpr_debug_newrecord", function(p)
    local dr = display_record
    
    display_record = nil
    
    onNewRecord(mpr.buildRecord(p.realtime, {p}, encoremode), {p})
    
    display_record = dr
end, COM_ADMIN)

COM_AddCommand("mpr_debug_setrecord", function(p)
	display_record = mpr.buildRecord(p.realtime, {p}, encoremode)
end)
--]]